<?php
namespace Intergo\AuthClient;

use Intergo\AuthClient\Requests\ApiClient\DeleteApiClientRequest;
use Intergo\AuthClient\Requests\ApiClient\EnableApiClientRequest;
use Intergo\AuthClient\Requests\ApiClient\GenerateApiClientRequest;
use Intergo\AuthClient\Requests\ApiClient\GetUserApiClientRequest;
use Intergo\AuthClient\Requests\ApiKey\DeleteApiKeyRequest;
use Intergo\AuthClient\Requests\ApiKey\DeleteUserApiKeysRequest;
use Intergo\AuthClient\Requests\ApiKey\DisableApiKeyRequest;
use Intergo\AuthClient\Requests\ApiKey\EnableApiKeyRequest;
use Intergo\AuthClient\Requests\ApiKey\GenerateApiKeyRequest;
use Intergo\AuthClient\Requests\ApiKey\GetUserApiKeysRequest;
use Intergo\AuthClient\Requests\Auth\AuthenticateRequest;
use Intergo\AuthClient\Requests\Auth\InvalidateUserRequest;
use Intergo\AuthClient\Requests\Auth\LoginByRequest;
use Intergo\AuthClient\Requests\Auth\VerifyEmailRequest;
use Intergo\AuthClient\Requests\Auth\InvalidateRequest;
use Intergo\AuthClient\Requests\Auth\LoginRequest;
use Intergo\AuthClient\Requests\Auth\RefreshRequest;
use Intergo\AuthClient\Requests\Auth\RegenerateVerificationCodeRequest;
use Intergo\AuthClient\Requests\Auth\RegisterRequest;
use Intergo\AuthClient\Requests\Auth\VerifyRequest;
use Intergo\AuthClient\Structs\Auth\LoginBy;
use Intergo\BaseClient\Responses\BaseResponseInterface;
use Intergo\AuthClient\Structs\ApiClient\DeleteApiClient;
use Intergo\AuthClient\Structs\ApiClient\DisableApiClient;
use Intergo\AuthClient\Structs\ApiClient\EnableApiClient;
use Intergo\AuthClient\Structs\ApiClient\GenerateApiClient;
use Intergo\AuthClient\Structs\ApiClient\GetUserApiClients;
use Intergo\AuthClient\Structs\ApiKey\DeleteApiKey;
use Intergo\AuthClient\Structs\ApiKey\DeleteUserApiKeys;
use Intergo\AuthClient\Structs\ApiKey\DisableApiKey;
use Intergo\AuthClient\Structs\ApiKey\EnableApiKey;
use Intergo\AuthClient\Structs\ApiKey\GenerateApiKey;
use Intergo\AuthClient\Structs\ApiKey\GetUserApiKeys;
use Intergo\AuthClient\Structs\Auth\Authenticate;
use Intergo\AuthClient\Structs\Auth\InvalidateUser;
use Intergo\AuthClient\Structs\Auth\VerifyEmail;
use Intergo\AuthClient\Structs\Auth\Invalidate;
use Intergo\AuthClient\Structs\Auth\Login;
use Intergo\AuthClient\Structs\Auth\Refresh;
use Intergo\AuthClient\Structs\Auth\RegenerateVerificationCode;
use Intergo\AuthClient\Structs\Auth\Register;
use Intergo\AuthClient\Structs\Auth\Verify;
use Intergo\BaseClient\Client;

class AuthClient extends Client
{
    /** ########### AUTHENTICATION ########### */

    /**
     * @param Register $register
     * @return BaseResponseInterface
     */
    public function register(Register $register)
    {
        $request = new RegisterRequest($this->baseUrl);
        return $this->processRequest($request, $register);
    }

    /**
     * @param Login $login
     * @return BaseResponseInterface
     */
    public function login(Login $login)
    {
        $request = new LoginRequest($this->baseUrl);
        return $this->processRequest($request, $login);
    }

    /**
     * @param LoginBy $login
     * @return BaseResponseInterface
     */
    public function loginBy(LoginBy $login)
    {
        $request = new LoginByRequest($this->baseUrl);
        return $this->processRequest($request, $login);
    }

    /**
     * @param Verify $authenticate
     * @return BaseResponseInterface
     */
    public function verify(Verify $authenticate)
    {
        $request = new VerifyRequest($this->baseUrl);
        return $this->processRequest($request, $authenticate);
    }

    /**
     * @param Authenticate $authenticate
     * @return BaseResponseInterface
     */
    public function authenticate(Authenticate $authenticate)
    {
        $request = new AuthenticateRequest($this->baseUrl);
        return $this->processRequest($request, $authenticate);
    }

    /**
     * @param Refresh $refresh
     * @return BaseResponseInterface
     */
    public function refresh(Refresh $refresh)
    {
        $request = new RefreshRequest($this->baseUrl);
        return $this->processRequest($request, $refresh);
    }

    /**
     * @param Invalidate $invalidate
     * @return BaseResponseInterface
     */
    public function invalidate(Invalidate $invalidate)
    {
        $request = new InvalidateRequest($this->baseUrl);
        return $this->processRequest($request, $invalidate);
    }

    /**
     * @param RegenerateVerificationCode $regenerateVerificationCode
     * @return BaseResponseInterface
     */
    public function regenerateVerificationCode(RegenerateVerificationCode $regenerateVerificationCode)
    {
        $request = new RegenerateVerificationCodeRequest($this->baseUrl);
        return $this->processRequest($request, $regenerateVerificationCode);
    }

    /**
     * @param VerifyEmail $verifyEmail
     * @return BaseResponseInterface
     */
    public function verifyEmail(VerifyEmail $verifyEmail)
    {
        $request = new VerifyEmailRequest($this->baseUrl);
        return $this->processRequest($request, $verifyEmail);
    }

    /**
     * @param InvalidateUser $invalidate
     * @return BaseResponseInterface
     */
    public function invalidateUser(InvalidateUser $invalidate)
    {
        $request = new InvalidateUserRequest($this->baseUrl);
        return $this->processRequest($request, $invalidate);
    }

    /** ########### API KEYS ########### */

    /**
     * @param GetUserApiKeys $getUserApiKeys
     * @return BaseResponseInterface
     */
    public function getUserApiKeys(GetUserApiKeys $getUserApiKeys)
    {
        $request = new GetUserApiKeysRequest($this->baseUrl);
        return $this->processRequest($request, $getUserApiKeys, 'GET');
    }

    /**
     * @param GenerateApiKey $generateApiKey
     * @return BaseResponseInterface
     */
    public function generateApiKey(GenerateApiKey $generateApiKey)
    {
        $request = new GenerateApiKeyRequest($this->baseUrl);
        return $this->processRequest($request, $generateApiKey);
    }

    /**
     * @param DisableApiKey $disableApiKey
     * @return BaseResponseInterface
     */
    public function disableApiKey(DisableApiKey $disableApiKey)
    {
        $request = new DisableApiKeyRequest($this->baseUrl);
        return $this->processRequest($request, $disableApiKey);
    }

    /**
     * @param EnableApiKey $enableApiKey
     * @return BaseResponseInterface
     */
    public function enableApiKey(EnableApiKey $enableApiKey)
    {
        $request = new EnableApiKeyRequest($this->baseUrl);
        return $this->processRequest($request, $enableApiKey);
    }

    /**
     * @param DeleteApiKey $deleteApiKey
     * @return BaseResponseInterface
     */
    public function deleteApiKey(DeleteApiKey $deleteApiKey)
    {
        $request = new DeleteApiKeyRequest($this->baseUrl);
        return $this->processRequest($request, $deleteApiKey);
    }

    /**
     * @param DeleteUserApiKeys $deleteUserApiKeys
     * @return BaseResponseInterface
     */
    public function deleteUserApiKeys(DeleteUserApiKeys $deleteUserApiKeys)
    {
        $request = new DeleteUserApiKeysRequest($this->baseUrl);
        return $this->processRequest($request, $deleteUserApiKeys);
    }

    /** ########### API CLIENTS ########### */

    /**
     * @param GetUserApiClients $getUserApiClients
     * @return BaseResponseInterface
     */
    public function getUserApiClients(GetUserApiClients $getUserApiClients)
    {
        $request = new GetUserApiClientRequest($this->baseUrl);
        return $this->processRequest($request, $getUserApiClients, 'GET');
    }

    /**
     * @param GenerateApiClient $generateApiClient
     * @return BaseResponseInterface
     */
    public function generateApiClient(GenerateApiClient $generateApiClient)
    {
        $request = new GenerateApiClientRequest($this->baseUrl);
        return $this->processRequest($request, $generateApiClient);
    }

    /**
     * @param DisableApiClient $disableApiClient
     * @return BaseResponseInterface
     */
    public function disableApiClient(DisableApiClient $disableApiClient)
    {
        $request = new DisableApiKeyRequest($this->baseUrl);
        return $this->processRequest($request, $disableApiClient);
    }

    /**
     * @param EnableApiClient $enableApiClient
     * @return BaseResponseInterface
     */
    public function enableApiClient(EnableApiClient $enableApiClient)
    {
        $request = new EnableApiClientRequest($this->baseUrl);
        return $this->processRequest($request, $enableApiClient);
    }

    /**
     * @param DeleteApiClient $deleteApiClient
     * @return BaseResponseInterface
     */
    public function deleteApiClient(DeleteApiClient $deleteApiClient)
    {
        $request = new DeleteApiClientRequest($this->baseUrl);
        return $this->processRequest($request, $deleteApiClient);
    }
}
